package com.xfront.owl;

import javax.xml.transform.TransformerFactory;
import javax.xml.transform.Transformer;
import javax.xml.transform.stream.StreamSource;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerConfigurationException;

import java.io.*;
import java.util.*;

public class TransformHelper 
{
	private Hashtable parameters = null;

	public TransformHelper() 
	{
		parameters = new Hashtable();
	}

	public void setParameter(String name, Object value) 
	{
		parameters.put(name, value);
	}

	public void removeAllParameters() 
	{
		parameters = new Hashtable();
	}

	/**
	  * The code below does the transformation, using any parameters present in
	  * the harshtable.  After doing so it deletes all parameters in the hashtable.
	  */
	public String doTransform(String xml, String xsl) throws TransformerException, TransformerConfigurationException, 
		   FileNotFoundException, IOException 
	{
		TransformerFactory tFactory = TransformerFactory.newInstance();
		StringReader xsl_sr = new StringReader(xsl);
		StreamSource xsl_ss = new StreamSource(xsl_sr);
		Transformer transformer = tFactory.newTransformer(xsl_ss);

		Enumeration paramNames = parameters.keys();
		while (paramNames.hasMoreElements()) 
		{
			String name = (String)paramNames.nextElement();
			Object value = parameters.get(name);
			transformer.setParameter(name, value);
		}

		StringReader xml_sr = new StringReader(xml);
		StreamSource xml_ss = new StreamSource(xml_sr);
		StringWriter out_sw = new StringWriter();
		StreamResult out_sr = new StreamResult(out_sw);
		transformer.transform(xml_ss, out_sr);

		parameters = new Hashtable();

		return out_sw.toString();
	}

	public String doTransform()
	throws TransformerException, TransformerConfigurationException, 
		   FileNotFoundException, IOException 
	{
		String xsl = this.getFile("OWL-API.xsl");
		String xml = "<?xml version=\"1.0\"?><Test/>";
		String result = this.doTransform(xml, xsl); 
		return result;
	}

	private String getFile(String filename) 
	{
		File file = new File(filename);
		if (!file.exists())
			System.err.println("no such file: " + filename);
		if (!file.isFile())
			System.err.println("invalid file (it is a directory): " + filename);
		if (!file.canRead())
			 System.err.println("file is unreadable: " + filename);

		// If we've gotten this far, then everything is okay.
		FileInputStream in = null;
		BufferedReader buffer = null;
		StringBuffer sb = new StringBuffer(2048);    
		String s = null;    
		try {
			in = new FileInputStream(file);
			buffer = new BufferedReader(new InputStreamReader(in));
			while ((s = buffer.readLine()) != null) {
				 sb = sb.append(s + "\n");
			}
		}
		catch (Exception e) { e.printStackTrace(); }
		finally {
			if (in != null)
				try {
					in.close();
				} catch (IOException e) { ; }
		}
		return sb.toString();
	}

	private void writeFile(String filename, String s) 
	{
		try 
		{
			FileWriter fw = new FileWriter(filename);
			BufferedWriter bw = new BufferedWriter(fw);
			bw.write(s, 0, s.length());
			bw.flush();
		}
		catch (IOException ioe) 
		{ ioe.printStackTrace(); }
	}

	/** unit testing **/
	public static void main(String[] args) 
	{
	   TransformHelper transformer = new TransformHelper();

	   String xml = transformer.getFile(args[0]);
	   String xsl = transformer.getFile(args[1]);
	   String out_filename = args[2];

	   int idx = 3;
	   boolean moreParams = true;
	   while (moreParams) 
	   {
		   if (args.length == idx) moreParams = false;
		   else if (args.length == idx+1) moreParams = false;
		   else 
		   {
			  String paramName = args[idx]; 
			  String paramValue = args[idx+1]; 
			  transformer.setParameter(paramName, paramValue); 
		   }
		   idx = idx + 2;
	   }

	   try 
	   {
		   String result = transformer.doTransform(xml, xsl); 
		   transformer.writeFile(out_filename, result);
	   } catch (Exception e) {e.printStackTrace();}
	}
}